# corridorbuild.py
#
# This script will create the transect corridors for a layer of transects
# for use with the adaptive sampling applets. The corridor will be the width
# of one cell and will be centered on the transect.
#
# Created by: Curt Swets
# 7/26/2010
#
# Import the necessary modules:
import arcgisscripting, math, os, copy, sys
#
# Create the goeprocessor
gp=arcgisscripting.create(9.3)
gp.overwriteoutput=1

# ARGUMENTS AND VARIABLES ##################

cellsize1= sys.argv[1]
cellsize=int(cellsize1)
linelayer= sys.argv[2]
outlayer= sys.argv[3]

############################################

# Define some functions for use at a later point in the script:        
def calcBearing(pnt1, pnt2):
    delx = pnt2.x - pnt1.x
    dely = pnt2.y - pnt1.y
    if delx < 0.0:
        bearing = math.radians(180) + math.atan(dely/delx)
    elif delx == 0.0:
        if dely < 0.0:
            bearing = math.radians(270)
        else:
            bearing = math.radians(90)
    else:
        if dely < 0.0:
            bearing = math.radians(360) + math.atan(dely/delx)
        else:
            bearing = math.atan(dely/delx)
    return bearing

def calcBearing(pnt1x, pnt1y, pnt2x, pnt2y):
    delx = pnt2x - pnt1x
    dely = pnt2y - pnt1y
    if delx < 0.0:
        bearing = math.radians(180) + math.atan(dely/delx)
    elif delx == 0.0:
        if dely < 0.0:
            bearing = math.radians(270)
        else:
            bearing = math.radians(90)
    else:
        if dely < 0.0:
            bearing = math.radians(360) + math.atan(dely/delx)
        else:
            bearing = math.atan(dely/delx)
    return bearing

def newXY(pnt1, pnt2, r):
    delx = pnt2.x - pnt1.x
    dely = pnt2.y - pnt1.ya
    newX = pnt1.x + r*math.cos(calcBearing(pnt1, pnt2))
    newY = pnt1.y + r*math.sin(calcBearing(pnt1, pnt2))
    return (newX,newY)

def newLongXY(pnt, theBearing, r):
    newX = pnt.x + r*math.cos(theBearing)
    newY = pnt.y + r*math.sin(theBearing)
    return (newX,newY)

def newStartXY(pnt, theBearing, r):
    newX = pnt.x + r*math.sin(math.pi/2 - theBearing)
    newY = pnt.y + r*math.cos(math.pi/2 - theBearing)
    return (newX,newY)

################### THE PROGRAM STARTS HERE ################################

# Start a searchcursor to find the start and end points of each transect as well
# as the length and define variables for all:
rows=gp.searchcursor(linelayer)
row=rows.next()
while row:
    start= row.shape.firstpoint
    end= row.shape.lastpoint
    transectLength=row.shape.length
    TransxId=row.id
    stx=start.x
    sty=start.y
    enx=end.x
    eny=end.y
            
# Calculate the mathematical bearing of the transect:
    transectBearing = calcBearing(enx, eny, stx, sty)
    
# Calculate the perpendicular bearing for the transect:
    if math.degrees(transectBearing) <270:
        Perp1=math.degrees(transectBearing)+90
        Perp2=0
        PerpBearing=Perp1+Perp2
    if not math.degrees(transectBearing) <270:
        Perp2=math.degrees(transectBearing)-90
        Perp1=0
        PerpBearing=Perp1+Perp2
        
# Caluclate the reverse bearing of the perpendicular bearing:
    if PerpBearing<180:
        RevPerp=PerpBearing+180
    elif PerpBearing==180:
        RevPerp=0
    elif PerpBearing>180:
        RevPerp=PerpBearing-180

# Create the point objects at the corners of the survey grid for the start of the transect:
    ospnt=newLongXY(start, math.radians(PerpBearing), (cellsize/2))
    ospnt1=newLongXY(start, math.radians(RevPerp), (cellsize/2))
    
# Create the point objects at the corners of the survey grid for the end of the transect:
    ospnt2=newLongXY(end, math.radians(PerpBearing), (cellsize/2))
    ospnt3=newLongXY(end, math.radians(RevPerp), (cellsize/2))

# Create polygon features:

# Define the location of the survey corridor dataclass:
    TransCor = (outlayer)
    
# Open insertcursor:
    outRows = gp.InsertCursor(TransCor)
    
# Create points:
    ll = gp.CreateObject("Point")
    lr = gp.CreateObject("Point")
    ul = gp.CreateObject("Point")
    ur = gp.CreateObject("Point")

    polyID = TransxId
    
# Make polygon features and insert into table:
    ll.x = ospnt[0]
    ll.y = ospnt[1]

    lr.x = ospnt1[0]
    lr.y = ospnt1[1]

    ul.x = ospnt2[0]
    ul.y = ospnt2[1]

    ur.x = ospnt3[0]
    ur.y = ospnt3[1]
            
# Add point objects to array - note ll is added twice b/c you need to close polygon:
    arrayObj = gp.CreateObject("Array")
    arrayObj.Add(ll)
    arrayObj.Add(lr)
    arrayObj.Add(ur)
    arrayObj.Add(ul)
    arrayObj.Add(ll)

# Create new row for output feature:
    outfeat = outRows.NewRow()
            
# Set feature ID:
    outfeat.id = polyID
            
# Assign array of points to output feature:
    outfeat.Shape = arrayObj
            
# Insert the feature:
    outRows.InsertRow(outfeat)
            
# Clear array of points:
    arrayObj.RemoveAll()
    polyID = polyID+1

#Clean up:
    del outRows, outfeat  

    
    row=rows.next()

    
