# transectbuild.py
#
# This script will create features in the geodatase feature class called
# surveyGrid for use with the adaptive sampling applet. This script requires
# a layer of line features, also in the geodatabase, called transectline.
# This line featureclass must be provided for this script to function properly. 
#
# Created by: Curt Swets
# 7/23/2010
#
#
# Import the necessary modules:
import arcgisscripting, math, os, copy, sys
#
# Create the Geoprocessor object
gp=arcgisscripting.create(9.3)
gp.overwriteoutput=1

# ARGUMENTS AND VARIABLES ##################

cellsize1=sys.argv[1]
cellsize=int(cellsize1)
NumCells=sys.argv[2]
SurveyWidth2=int(NumCells)*cellsize
SurveyWidth=SurveyWidth2*2
inline=sys.argv[3]
outgrid=sys.argv[4]

############################################

# Define some functions for use at a later point in the script:        
def calcBearing(pnt1, pnt2):
    delx = pnt2.x - pnt1.x
    dely = pnt2.y - pnt1.y
    if delx < 0.0:
        bearing = math.radians(180) + math.atan(dely/delx)
    elif delx == 0.0:
        if dely < 0.0:
            bearing = math.radians(270)
        else:
            bearing = math.radians(90)
    else:
        if dely < 0.0:
            bearing = math.radians(360) + math.atan(dely/delx)
        else:
            bearing = math.atan(dely/delx)
    return bearing

def calcBearing(pnt1x, pnt1y, pnt2x, pnt2y):
    delx = pnt2x - pnt1x
    dely = pnt2y - pnt1y
    if delx < 0.0:
        bearing = math.radians(180) + math.atan(dely/delx)
    elif delx == 0.0:
        if dely < 0.0:
            bearing = math.radians(270)
        else:
            bearing = math.radians(90)
    else:
        if dely < 0.0:
            bearing = math.radians(360) + math.atan(dely/delx)
        else:
            bearing = math.atan(dely/delx)
    return bearing

def newXY(pnt1, pnt2, r):
    delx = pnt2.x - pnt1.x
    dely = pnt2.y - pnt1.ya
    newX = pnt1.x + r*math.cos(calcBearing(pnt1, pnt2))
    newY = pnt1.y + r*math.sin(calcBearing(pnt1, pnt2))
    return (newX,newY)

def newLongXY(pnt, theBearing, r):
    newX = pnt.x + r*math.cos(theBearing)
    newY = pnt.y + r*math.sin(theBearing)
    return (newX,newY)

def newStartXY(pnt, theBearing, r):
    newX = pnt.x + r*math.sin(math.pi/2 - theBearing)
    newY = pnt.y + r*math.cos(math.pi/2 - theBearing)
    return (newX,newY)

################### THE PROGRAM STARTS HERE ################################

# Adaptive Web applet requires ROW and COL fields. Create them if they do
# not exist. Adaptive Cluster applet will run with ROW and COL fields so in
# order to have one script for both applets each GDB will include these fields.
descript = gp.describe(outgrid)
myfields = descript.fields
for item in myfields:
    if not item.name == 'row':
        gp.addfield_management(outgrid, 'row','short')
    if not item.name == 'col':
        gp.addfield_management(outgrid, 'col','short')
    
# Start a searchcursor to find the start and end points of each transect as well
# as the length and define variables for all:
rows=gp.searchcursor(inline)
row=rows.next()
while row:
    start= row.shape.firstpoint
    end= row.shape.lastpoint
    transectLength=int(math.ceil(row.shape.length))
    TransxId=row.id
    stx=start.x
    sty=start.y
    enx=end.x
    eny=end.y
            
# Calculate the mathematical bearing of the transect:
    transectBearing = calcBearing(enx, eny, stx, sty)
    
# Calculate the perpendicular bearing for the transect:
    if math.degrees(transectBearing) <270:
        Perp1=math.degrees(transectBearing)+90
        Perp2=0
        PerpBearing=Perp1+Perp2
    if not math.degrees(transectBearing) <270:
        Perp2=math.degrees(transectBearing)-90
        Perp1=0
        PerpBearing=Perp1+Perp2
        
# Caluclate the reverse bearing of the perpendicular bearing:
    if PerpBearing<180:
        RevPerp=PerpBearing+180
    elif PerpBearing==180:
        RevPerp=0
    elif PerpBearing>180:
        RevPerp=PerpBearing-180

# Create the point objects at the corners of the survey grid for the start of the transect:
    ospnt=newLongXY(start, math.radians(PerpBearing), (SurveyWidth/2)+(cellsize/2))
    ospnt1=newLongXY(start, math.radians(RevPerp), (SurveyWidth/2)+cellsize/2)
    
# Create the point objects at the corners of the survey grid for the end of the transect:
    ospnt2=newLongXY(end, math.radians(PerpBearing), (SurveyWidth/2)+(cellsize/2))
    ospnt3=newLongXY(end, math.radians(RevPerp), (SurveyWidth/2)+cellsize/2)
    
# Define the number of rows and columns based on transect length and cellsize:   
    numRows = (transectLength/cellsize)
    numCols =  ((SurveyWidth) + cellsize)/cellsize

# Create the point objects needed:
    curPnt = gp.CreateObject("Point")
    nextPnt = gp.CreateObject("Point")
    startPnt = gp.CreateObject("Point")

# Fill a list with equally spaced (cellSize) coordinates. There will be one list for
# each line parallel to transect:

# Set the start point:
    startPnt.X = ospnt2[0]
    startPnt.Y = ospnt2[1]

# Create some loop variables:
    columns = []
    curPnt.X = ospnt2[0]
    curPnt.Y = ospnt2[1]
    columns.insert(0,[float(curPnt.X), float(curPnt.Y)])
    vertexMatrix = []

# Loop and fill 2-D list with coordinates
    for i in range (0,int(numCols+1)):
        for j in range (1,int(numRows+1)):
            nextPnt.X, nextPnt.Y = newLongXY(curPnt, transectBearing,cellsize)
            columns.insert(j, [float(nextPnt.X), float(nextPnt.Y)])
            curPnt.X = float(nextPnt.X)
            curPnt.Y = float(nextPnt.Y)
        vertexMatrix.insert(i, copy.deepcopy(columns))
        del columns[:]
        curPnt.X, curPnt.Y = newStartXY(startPnt, math.radians(RevPerp), cellsize*(i+1))
        columns.insert(0,[float(curPnt.X), float(curPnt.Y)])

# Create polygon features:

# Define the location of the surveyGrid dataclass:
    surveyGridPoly = (outgrid)

# Open insertcursor:
    outRows = gp.InsertCursor(surveyGridPoly)
    
# Create points:
    ll = gp.CreateObject("Point")
    lr = gp.CreateObject("Point")
    ul = gp.CreateObject("Point")
    ur = gp.CreateObject("Point")

    polyID = 0
    
# Make polygon features and insert into table:
    for j in range (0, (len(vertexMatrix)-1)):
           for i in range (0, (len(vertexMatrix[j])-1)):
            ll.x = vertexMatrix[j][i][0]
            ll.y = vertexMatrix[j][i][1]

            lr.x = vertexMatrix[j+1][i][0]
            lr.y = vertexMatrix[j+1][i][1]

            ul.x = vertexMatrix[j][i+1][0]
            ul.y = vertexMatrix[j][i+1][1]

            ur.x = vertexMatrix[j+1][i+1][0]
            ur.y = vertexMatrix[j+1][i+1][1]
            
# Add point objects to array - note ll is added twice b/c you need to close polygon:
            arrayObj = gp.CreateObject("Array")
            arrayObj.Add(ll)
            arrayObj.Add(lr)
            arrayObj.Add(ur)
            arrayObj.Add(ul)
            arrayObj.Add(ll)

# Create new row for output feature:
            outfeat = outRows.NewRow()
            
# Set feature ID:
            outfeat.id = polyID
            outfeat.row = i
            outfeat.col = j
            outfeat.status = 'Not Sampled'
            outfeat.scode = '2'
# Assign array of points to output feature:
            outfeat.Shape = arrayObj
            
# Insert the feature:
            outRows.InsertRow(outfeat)
            
# Clear array of points:
            arrayObj.RemoveAll()
            polyID = polyID+1

#Clean up:
    del outRows, outfeat  

    
    row=rows.next()
